/**
*	@author: Thomas Brandého
*/

#ifndef INPUT_H
#define INPUT_H
#include "keybindings.h"
#include <queue>
#include <vector>
#include <map>
#include <unordered_map>
#include "textbuffer.h"
#include "glm/vec2.hpp"

class Input{
private:
    /* window-related variables */
    sf::Window* m_window;
    bool m_closeRequested;
    bool m_hasBeenResized;
    bool m_imgui_enabled;

    /* general action-mapping variables */
    IKeysMap m_keysmap;
    std::vector<Action> m_actions;
    int nb_actions;

    /* context-related variables */
    std::string m_current_context;
    std::vector<Context> m_contexts;
    std::unordered_map<std::string,KeyBindings> m_keybindings;

    /* keyboard */
    std::vector<int> m_heldKeys;
    void releaseHeldKeys(int keycode);

    /* mouse */
    sf::Vector2i m_mouse_position;
    sf::Vector2i m_last_mouse_position;
    float m_delta_vertical_scroll;
    bool m_mouseGrabbed;
    bool m_mouseWasGrabbed;
    std::vector<int> m_heldMouseButtons;
    void releaseHeldMouseButton(int buttoncode);

    /* joystick */
    std::map<int,std::map<int,float>> m_joystick_position;
    std::map<int,std::vector<int>> m_heldJoystickButtons;
    void releaseHeldJoystickButton(int buttoncode, int controller_id);

    /* text-related variable */
    std::wstring m_buffer;
public:
    /* Constructors */
    Input(sf::Window *w);

    /* general action-mapping functions */
    void setKeysMap(IKeysMap km);  //set bindings
    IKeysMap getKeysMap(){return m_keysmap;}
    void updateEvents();           //handle the input and put the associated event in the file
    std::vector<Action> getActions();               //get the first action in the file of event

    /* context-related functions */
    void addContext(Context context){m_contexts.push_back(context); updateKeyBindings();}
    std::string getCurrentContext(){return m_current_context;}
    void setCurrentContext(std::string context_name){m_current_context = context_name;}
    std::vector<std::string> getContextsList();
    void updateKeyBindings();

    /* window-related function */
    bool isCloseRequested() const;
    bool isResized() const;
    void enableImgui();
    void disableImgui();

    /* keyboard-related functions */
    bool isKeyPressed(int key) const;

    /* mouse-related function */
    glm::vec2 getPosition() const;
    glm::vec2 getDeltaPosition() const;
    float getDeltaVerticalScroll() const;

    /* Mouse grabbing */
    void setMouseGrabbed(bool isGrabbed);
    void toggleMouseGrab() { setMouseGrabbed(!m_mouseGrabbed); }
    bool isMouseGrabbed() const { return m_mouseGrabbed; }

    /* joystick-related function */
    std::vector<int> getControllersConnected() const;
    float getAxisPosition(int device_id, int axis_id);

    /* text-related function */
    std::wstring getText();

    void test();
};

#endif //INPUT_H
