/**
*	@author: Thomas Brandého
*/

#ifndef KEYBINDINGS_H
#define KEYBINDINGS_H

#include <SFML/Window.hpp>

#include <cereal/archives/json.hpp>
#include <cereal/types/vector.hpp>
#include <cassert>

#include <unordered_map>

#define NO_KEY -1
#define NO_ACTION -1

namespace input{
enum Source{NONE=-1,KEYBOARD,MOUSE,CONTROLLER,SOURCE_COUNT};
enum Controller{BUTTON_A,BUTTON_B,BUTTON_X,BUTTON_Y,BUTTON_LB,BUTTON_RB,BUTTON_SELECT,BUTTON_START,BUTTON_L3, BUTTON_R3};
enum Axis{LEFT_JOYSTICK_HORIZONTAL,LEFT_JOYSTICK_VERTICAL,LT_RT,RIGHT_JOYSTICK_VERTICAL,RIGHT_JOYSTICK_HORIZONTAL,DPAD_HORIZONTAL=6,DPAD_VERTICAL};
}

struct Action
{
    int action;
    int source;
    int controller_id;
    static Action getNull();
    bool isNull(){
        return (action == NO_ACTION) && (source == input::NONE);
    }
    friend bool operator ==(const Action& action1, const Action& action2){
        return (action1.action == action2.action) && (action1.source == action2.source);
    }
    Action& operator=(const Action& newAction);
    Action(){}
    Action(std::initializer_list<int> c){
        assert(c.size() <= 3);
        int array[3];
        std::copy(c.begin(),c.end(),array);
        switch (c.size()) {
        case 3:
            controller_id = array[2];
        case 2:
            source = array[1];
        case 1:
            action = array[0];
            break;
        default:
            break;
        } 
    }

    template <class Archive>
    void serialize(Archive & archive)
    {
        archive(CEREAL_NVP(action),CEREAL_NVP(source),CEREAL_NVP(controller_id));
    }
};

struct Binding// : public Serializable
{
    Action action;
    int key;
    int type;
    Binding& operator=(const Binding newBinding);

    template <class Archive>
    void serialize(Archive & archive)
    {
        archive(CEREAL_NVP(action),CEREAL_NVP(key),CEREAL_NVP(type));
    }
};

class IKeysMap
{
public:
	enum {PRESSED, RELEASED, HOLD};
	IKeysMap();
    std::vector<Binding> getBindings(Action action) const;
    std::vector<Binding>& data(){return keys;}
protected:
    std::vector<Binding> keys;

    friend class cereal::access;
    template <class Archive>
    void serialize(Archive & archive)
    {
        archive(CEREAL_NVP(keys));
    }
};

class Context {
	public:
        Context(std::string name, std::vector<Action> actions);
		std::string getName();
        std::vector<Action> getActions() const;
	private:
		std::string name;
        std::vector<Action> actions;
};

class KeyBindings {
	public:
		KeyBindings();
		KeyBindings(const Context &context, const IKeysMap &keysmap);
        Action getPressedAction(input::Source src,int key) const;
        Action getReleasedAction(input::Source src,int key) const;
        Action getHoldAction(input::Source src,int key) const;

	private:
        std::unordered_map<int,std::unordered_map<int,int>> bindings_pressed;
        std::unordered_map<int,std::unordered_map<int,int>> bindings_released;
        std::unordered_map<int,std::unordered_map<int,int>> bindings_hold;
        void setPressedAction(int key, Action action);
        void setReleasedAction(int key, Action action);
        void setHoldAction(int key, Action action);
        void init_map_bindings();
};

#endif
